/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#include <project.h>
#include <stdio.h>

#define RxBufferSize    128     //Size of the RX circular buffer    
#define EOM_CR          0x0D    //End-of-message delimiter (Carriage Return)

// Status values for commands SENT from (this) host
#define TX_COMMAND_STATUS_PENDING_RESPONSE  1
#define TX_COMMAND_STATUS_SUCCESS           2
#define TX_COMMAND_STATUS_CRC_FAILED        3
#define TX_COMMAND_STATUS_SYNTAX_ERROR      4
#define TX_COMMAND_STATUS_RTS               5
#define TX_COMMAND_STATUS_RETRY             6


// Commands that can be RECEIVED from the display
#define COMMAND_1	"c1"
#define COMMAND_2	"c2"
#define COMMAND_3   "c3"


typedef unsigned char BYTE;


// Variables for TX
char cmdStr[64]; //  Array used to prep and hold commands for TX.
BYTE txCmdStatus; // Command status

// Variables for RX
char RxStr[64];  // Received Rx packet (to hold individual command message)
uint8	RxBuffer[RxBufferSize];     // Rx circular buffer to hold all incoming command
uint8  *RxReadIndex	 = RxBuffer;    // pointer to position in RxBuffer to read and process bytes
uint8  *RxWriteIndex = RxBuffer;    // pointer to position in RxBuffer to write incoming Rx bytes
char   *RxStrIndex = RxStr;         // pointer to command string buffer (processed messages)


// Function Declarations ( Their definitions are after main(). )
void SendTXCommandWait( void );  // Sends TX commands to UART and waits for success/error response from display
void GetRxStr(uint8 c);          // Interupt handler called from ISR in UART_1_INT.c, line 180
void ProcessCommandPrompt( char * strCommand ); // Determines whether command prompt received indicates success or failure
void DispatchCommand(char * strCommand);  // Command handler for commands received from display


volatile int process_command_flag = 0 ;
volatile int dispatch_command_flag = 0 ;

int main()
{   
    /* Enable global interrupts. */
    CyGlobalIntEnable; 
        
    // Initialize UART
    UART_1_Start();
    
    // Clear RX and TX buffers
    UART_1_ClearRxBuffer();
    UART_1_ClearTxBuffer();
        
    // Set TX Command Buffer ready-to-send
    txCmdStatus = TX_COMMAND_STATUS_RTS;
    
    
    // Prepare 1st command to send to display
    sprintf( cmdStr, "This is the 1st command sent from host.\r" );
    // Send it to display and wait for success response (">").
    SendTXCommandWait();
    
    // Prepare 2nd command to send to display
    sprintf( cmdStr, "This is the 2nd command sent from host.\r" );
    // Send it to display and wait for success response (">").
    SendTXCommandWait();
    
    // Prepare 3rd command to send to display
    sprintf( cmdStr, "This is the 3rd command sent from host.\r" );
    // Send it to display and wait for success response (">").
    SendTXCommandWait();    
        
    for(;;)
    {   
        // Do lots of other stuff here.
        if (process_command_flag) {
            ProcessCommandPrompt(RxStr);
            process_command_flag = 0 ;
        }
        if (dispatch_command_flag) {
            DispatchCommand(RxStr);
            dispatch_command_flag = 0 ;
        }
    }
        
}

char pattern[] = { '|', '\\', '-', '/', '|', '\\', '-', '/' } ;
void cursor(void)
{
    static int phase = 0 ;
    UART_1_PutChar(0x08) ;
    UART_1_PutChar(pattern[phase]) ;
    CyDelay(100) ;
    phase = (phase + 1) % 9 ;
}

#if 0
void SendTXCommandWait( void )
{
    
    // Send prepared command (in cmdStr)
    UART_1_PutString( cmdStr );
    CyDelay(10) ; /* by moto */

    txCmdStatus = TX_COMMAND_STATUS_PENDING_RESPONSE;
    
    while ( txCmdStatus != TX_COMMAND_STATUS_RTS )
    {    
        if ( txCmdStatus == TX_COMMAND_STATUS_SUCCESS )
        {
            txCmdStatus = TX_COMMAND_STATUS_RTS;
            break;    
        }
    }
}
#else
void SendTXCommandWait( void )
{
    
    // Send prepared command (in cmdStr)
    UART_1_PutString( cmdStr );
    CyDelay(10) ; /* by moto */

    txCmdStatus = TX_COMMAND_STATUS_PENDING_RESPONSE;
    
    while ( txCmdStatus != TX_COMMAND_STATUS_RTS )
    {    
        switch(txCmdStatus) {
        case TX_COMMAND_STATUS_SUCCESS:
            txCmdStatus = TX_COMMAND_STATUS_RTS;
            break ;
        case TX_COMMAND_STATUS_PENDING_RESPONSE:
            cursor() ;
            // UART_1_PutString("Pending Response\n\r") ;
            // CyDelay(10) ;
            break ;
        case TX_COMMAND_STATUS_CRC_FAILED:
            UART_1_PutString("CRC Failed\n\r") ;
            CyDelay(10) ;
            break ;
        case TX_COMMAND_STATUS_SYNTAX_ERROR:
            UART_1_PutString("Syntax Error\n\r") ;
            CyDelay(10) ;
            break ;
        case TX_COMMAND_STATUS_RTS:
            UART_1_PutString("Status RTS received\n\r") ;
            break ;
        case TX_COMMAND_STATUS_RETRY:
            UART_1_PutString("Status Entry\n\r") ;
            break ;
        default:
            break ;
        }
    }
}    
#endif

void GetRxStr(uint8 rxchar) {
    
    uint8 c = rxchar;
    
    // If end-of-message command delimeter (Carriage Return) was received
    if ( c == EOM_CR )
    {
        // terminate string, excluding delimeter
        *RxStrIndex = 0;
        // reset pointer
        RxStrIndex = RxStr;
        
        // If a non-empty message was received
        if (strlen(RxStr) > 0)
        {
            // If a single character, then message is a command prompt only
            if ( strlen( RxStr ) == 1 )
            {
                // process_command_flag = 1 ;
                ProcessCommandPrompt(RxStr);
            }
            
            // If more than a single character, then message is a full command
            else
            {
                dispatch_command_flag = 1 ;
                // DispatchCommand(RxStr);
            }
        }  
    }
    
    // Otherwise, string body character was received...
    else
    {
        // build the command message
        *RxStrIndex++ = c;                  
    }    
}

void ProcessCommandPrompt( char * strCommand )
{
	
    char * strPtr;		// Pointer to command
	strPtr = strCommand;
	
	if( strcmp(">", strPtr) == 0 )
	{
		// SUCCESS: Requested action completed.
        txCmdStatus = TX_COMMAND_STATUS_SUCCESS;
	}
	else if( strcmp("#", strPtr) == 0 )
	{
		// CRC FAILED: Command not processed.
        txCmdStatus = TX_COMMAND_STATUS_CRC_FAILED;
	}
	else if( strcmp("!", strPtr) == 0 )
	{
		// SYNTAX ERROR: Command not processed.
        txCmdStatus = TX_COMMAND_STATUS_SYNTAX_ERROR;
	}
    else
    {
        txCmdStatus = TX_COMMAND_STATUS_PENDING_RESPONSE;
    }

}

void DispatchCommand(char * strCommand)
{
	
    char * strPtr;		// Pointer to command
	strPtr = strCommand;

	// Check for Command 1 received
	if( strcmp(COMMAND_1, strCommand) == 0 )
    {  
        // Prepare response to send to display
        sprintf( cmdStr, "Response to Command 1\r" );
        // Send response
        SendTXCommandWait();
    }
    
	// Check for Command 2 received
	else if( strcmp(COMMAND_2, strCommand) == 0 )
    {
        // Prepare response to send to display
        sprintf( cmdStr, "Response to Command 2\r" );
        // Send response
        SendTXCommandWait();   
	}
	// Check for Command 3 received
	else if( strcmp(COMMAND_3, strCommand) ==0 )
    {
        // Prepare response to send to display
        sprintf( cmdStr, "Response to Command 3\r" );
        // Send response
        SendTXCommandWait();   
	}
}

/* [] END OF FILE */
